const { Reporter, CLI } = require("html-validate");

/**
 * Takes formatter data in any of the following formats:
 *
 * - string
 * - string[]
 * - [string, object][]
 *
 * The option object can contain the following options:
 *
 * - "dest": set output destination file
 */
function prepareFormatter(formatter) {
	if (!Array.isArray(formatter)) {
		formatter = [formatter];
	}

	return formatter
		.map((entry) => (Array.isArray(entry) ? entry : [entry])) // string -> string[]
		.map((entry) => (entry.length < 2 ? [entry[0], {}] : entry)) // [string] -> [string, object]
		.map(([name, options]) => {
			if (options.dest) {
				return `${name}=${options.dest}`;
			} else {
				return name;
			}
		})
		.join(",");
}

function getFormatter(cli, options) {
	const formatString = prepareFormatter(options.format);
	return cli.getFormatter(formatString);
}

module.exports = function (grunt) {
	grunt.registerMultiTask("htmlvalidate", "Validate HTML files", function () {
		const cli = new CLI();
		const htmlvalidate = cli.getValidator();
		const options = this.options({
			failOnError: true,
			format: "stylish",
		});
		const formatter = getFormatter(cli, options);
		const fn = "validateFileSync" in htmlvalidate ? "validateFileSync" : "validateFile";
		const validateFile = (filename) => htmlvalidate[fn](filename);

		let results = [];
		this.files.forEach((fileset) => {
			results = results.concat(fileset.src.map(validateFile));
		});

		const merged = Reporter.merge(results);
		const output = formatter(merged);
		if (output) {
			grunt.log.write(output);
		}

		return merged.valid || !options.failOnError;
	});
};
